var Histology = function()
{
	var histology = {};
	var debug = 0;
	
	var nameIsValid = true;
	var histologyIsValid = false;
	var subtypeHistology = null;
	
	// Page initialization
	histology.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', histology.validateStep1);
		Events.addEventHandler('name', 'change', histology.nameOnChange);
		Events.addEventHandler('histology', 'change', histology.histologyOnChange);
		Buttons.addClickHandler('btnSelectHistologyItems', histology.selectHistologyItems);
		Events.addEventHandler('histology', 'base-selected', histology.onHistologySelected);
		
		// Navigation
		Buttons.addClickHandler('gocancel', histology.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('goprotocol', histology.showProtocol);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', histology.submit);
		
		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnembeddedHistologyItems';
		Wizard.showLoadingAnimation('Loading unembedded histology items...');
		Wizard.asyncJsonRequest(url, histology.histologyItemsLoaded);
	}
	
	histology.histologyItemsLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var items = response.histology;
		
		var histologyList = frm.histology;
		if (items.length > 0)
		{
			for (var i=0; i < items.length; i++)
			{
				var his = items[i];
				var name = (i+1) + ': ' + his.name;
				if (his.bioWell)
				{
					name += ' -- ' + his.bioWell.bioPlate.name + ' (' + his.bioWell.location+')';
				}
				var option = new Option(name, his.id);
				option.histology = his;
				histologyList.options[histologyList.length] = option;
			}
			histology.histologyOnChange();
		}
		else
		{	
			Wizard.setInputStatus('histology', 'invalid', 'No Histology items available for processing.');
		}
		
		Doc.show('goregister');
		Doc.show('gocancel');
		Doc.show('step-1');
	}

	histology.nameOnChange = function()
	{
		var frm = document.forms['reggie'];
		var name = frm.name.value;
		
		nameIsValid = false;
		if (name == '')
		{
			Wizard.setInputStatus('name', 'invalid', 'Missing');
			return;
		}
		Wizard.setInputStatus('name', 'valid');
		nameIsValid = true;
	}
	
	histology.histologyOnChange = function()
	{
		histologyIsValid = false;
		
		var frm = document.forms['reggie'];
		var histologyList = frm.histology;
		
		var numSelected = 0;
		for (var i = 0; i < histologyList.length; i++)
		{
			if (histologyList[i].selected) numSelected++;
		}

		Doc.element('numSelected').innerHTML = numSelected + ' selected';
		
		if (numSelected == 0 || numSelected % 5 != 0)
		{
			Wizard.setInputStatus('histology', 'invalid', 'Must select items in blocks of 5.');
			return;
		}
		
		histologyIsValid = true;
		Wizard.setInputStatus('histology', 'valid');
	}

	histology.selectHistologyItems = function()
	{
		var frm = document.forms['reggie'];
		if (frm.histology.disabled) return;
		
		if (subtypeHistology == null)
		{
			subtypeHistology = Reggie.getSubtypeInfo('HISTOLOGY');
		}
		
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeHistology.id;
		url += '&tmpfilter:DATE:creationEvent.eventDate='+encodeURIComponent('=');
		Dialogs.selectItem('SAMPLE', 'histology', 1, url);
	}

	histology.onHistologySelected = function(event)
	{
		var his = event.detail;
		
		var frm = document.forms['reggie'];
		var histologyList = frm.histology;
		var isNew = true;
		for (var i = 0; i < histologyList.length; i++)
		{
			if (histologyList[i].value == his.id)
			{
				histologyList[i].selected = true;
				isNew = false;
			}
		}

		if (isNew)
		{
			var option = new Option(his.name, his.id, true, true);
			histologyList[histologyList.length] = option;
		}
		
		if (event.detail.remaining == 0)
		{
			histology.histologyOnChange();
		}
	}
	
	histology.validateStep1 = function(event)
	{
		var valid = nameIsValid;
		valid &= histologyIsValid;
		
		if (!valid) event.preventDefault();
	}

	histology.submit = function()
	{
		var frm = document.forms['reggie'];

		var submitInfo = {};
		submitInfo.name = frm.name.value;
		submitInfo.comments = frm.comments.value;
		
		var items = [];
		for (var i = 0; i < frm.histology.length; i++)
		{
			if (frm.histology[i].selected) 
			{
				items[items.length] = parseInt(frm.histology[i].value, 10);
			}
		}
		
		submitInfo.histology = items;

		var url = '../Histology.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateWorkList';		
		Wizard.showLoadingAnimation('Creating Histology work list...');
		Wizard.asyncJsonRequest(url, histology.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
		
	histology.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		
		Doc.show('gorestart');
		if (response.workList)
		{
			var printFrm = document.forms['print'];
			printFrm.workList.value = response.workList.id;
			Doc.show('goprotocol');
		}
	}
	
	histology.cancelWizard = function()
	{
		location.href = 'histology_protocol.jsp?ID='+App.getSessionId();
	}
	
	histology.showProtocol = function()
	{
		var printFrm = document.forms['print'];
		printFrm.submit();
	}

	return histology;
}();

Doc.onLoad(Histology.initPage);

